@@ -85,6 +85,10 @@ def geocode(ctx, geocoder):
         click.echo(f"Adding column: {longitude}")
         table.add_column(longitude, float)
 
+    if "geocoder" not in table.columns_dict:
+        click.echo("Adding geocoder column")
+        table.add_column("geocoder", str)
+
     # always use a rate limiter, even if delay is zero
     geocode = RateLimiter(geocoder.geocode, min_delay_seconds=delay)
 
@@ -93,6 +97,7 @@ def geocode(ctx, geocoder):
     )
 
     done = 0
+    errors = []
 
     gen = geocode_list(
         rows, geocode, location, latitude_column=latitude, longitude_column=longitude
@@ -105,9 +110,14 @@ def geocode(ctx, geocoder):
                 table.update(pks, row)
                 done += 1
             else:
-                click.echo(f"Error geocoding row: {pks}", err=True)
+                errors.append(pks)
 
     click.echo("Geocoded {} rows".format(done))
+    if errors:
+        click.echo("The following rows failed to geocode:")
+        for pk in errors:
+            row = table.get(pk)
+            click.echo(f"{pk}: {location.format(row)}")
 
 
 @cli.command("test", hidden=True)
@@ -180,3 +190,15 @@ def open_mapquest(ctx, database, table, location, delay, latitude, longitude, ap
     click.echo("Using MapQuest geocoder")
     fill_context(ctx, database, table, location, delay, latitude, longitude)
     return geocoders.MapQuest(api_key=api_key)
+
+
+@cli.command("mapbox")
+@common_options
+@click.option(
+    "-k", "--api-key", type=click.STRING, required=True, envvar="MAPBOX_API_KEY"
+)
+def mapbox(ctx, database, table, location, delay, latitude, longitude, api_key):
+    "Mapbox"
+    click.echo("Using Mapbox geocoder")
+    fill_context(ctx, database, table, location, delay, latitude, longitude)
+    return geocoders.MapBox(api_key=api_key)
